@@ -16,9 +16,11 @@
 ##' iff the list \code{x} is of length 0. \code{is.atom} returns
 ##' \code{TRUE} iff the list \code{x} is of length 1.
 ##' \code{is.composite} returns \code{TRUE} iff the list \code{x} is
-##' of length > 1.
+##' of length > 1. \code{contains} return \code{TRUE} iff the list
+##' \code{x} contains an element identical to \code{elt}.
 ##'
 ##' @param x A list or vector.
+##' @param elt An element of a list or vector.
 ##'
 ##' @rdname lispLists
 first <- function(x) x[[1]]
@@ -47,6 +49,12 @@ is.atom <- function(x) length(x) == 1
 ##' @rdname lispLists
 is.composite <- function(x) length(x) > 1
 
+##' @rdname lispLists
+contains <- function(x, elt) {
+  for (candidate in x) if (identical(candidate, elt)) return(TRUE)
+  FALSE
+}
+
 ##' Sort a vector or list by the result of applying a function
 ##'
 ##' Sorts a vector or a list by the numerical result of applying the function \code{byFunc}.
@@ -150,10 +158,21 @@ randelt <- function(x, prob = NULL) {
 ##' sublists (or groups).
 ##' \code{groupListDistributed} distributes \code{l} into \code{numberOfGroups}
 ##' sublists (or groups).
+##' \code{flatten} flattens a list \code{l} of lists into a flat list by concatenation. If
+##' \code{recursive} is \code{TRUE} (defaults to \code{FALSE}), flatten will be recursively
+##' called on each argument first.
+##' \code{intersperse} joins two lists \code{xs} and \code{ys} into a list of pairs containig
+##' every possible pair, i.e. \code{intersperse(xs, ys)} equals the product list of \code{xs}
+##' and \code{ys}. The \code{pairConstructor} parameter can be used to change the type of pairs
+##' returned.
 ##'
 ##' @param l A list.
+##' @param xs A list.
+##' @param ys A list.
+##' @param pairConstructor The function to use for constructing pairs, defaults to \code{list}.
 ##' @param groupAssignment A vector of group assignment indices.
 ##' @param numberOfGroups The number of groups to create, must be <= length(l)
+##' @param recursive Whether to operate recursively on sublists or vectors.
 ##' @return A list of lists, where each member represents a group.
 ##'
 ##' @rdname listSplittingAndGrouping
@@ -184,3 +203,14 @@ groupListDistributed <- function(l, numberOfGroups) {
   splitList(l, gassign)
 }
 
+##' @rdname listSplittingAndGrouping
+flatten <- function(l, recursive = FALSE)
+  if (recursive && !is.atom(l)) {
+    Reduce(function(a, b) c(flatten(a, recursive = TRUE), flatten(b, recursive = TRUE)), l, init = list()) 
+  } else {
+    Reduce(c, l, init = list())
+  }
+
+##' @rdname listSplittingAndGrouping
+intersperse <- function(xs, ys, pairConstructor = list)
+  Reduce(c, Map(function(x1) Map(function(x2) pairConstructor(x1, x2), ys), xs))
